/*******************************************************************************
* File Name: myUART.c
* Version 2.30
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "myUART.h"
#include "CyLib.h"
#if(myUART_INTERNAL_CLOCK_USED)
    #include "myUART_IntClock.h"
#endif /* End myUART_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 myUART_initVar = 0u;
#if( myUART_TX_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))
    volatile uint8 myUART_txBuffer[myUART_TXBUFFERSIZE];
    volatile uint8 myUART_txBufferRead = 0u;
    uint8 myUART_txBufferWrite = 0u;
#endif /* End myUART_TX_ENABLED */
#if( ( myUART_RX_ENABLED || myUART_HD_ENABLED ) && \
     (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH) )
    volatile uint8 myUART_rxBuffer[myUART_RXBUFFERSIZE];
    volatile uint8 myUART_rxBufferRead = 0u;
    volatile uint8 myUART_rxBufferWrite = 0u;
    volatile uint8 myUART_rxBufferLoopDetect = 0u;
    volatile uint8 myUART_rxBufferOverflow = 0u;
    #if (myUART_RXHW_ADDRESS_ENABLED)
        volatile uint8 myUART_rxAddressMode = myUART_RXADDRESSMODE;
        volatile uint8 myUART_rxAddressDetected = 0u;
    #endif /* End EnableHWAddress */
#endif /* End myUART_RX_ENABLED */


/*******************************************************************************
* Function Name: myUART_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The myUART_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time UART_Start() is called. This allows for
*  component initialization without re-initialization in all subsequent calls
*  to the myUART_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void myUART_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(myUART_initVar == 0u)
    {
        myUART_Init();
        myUART_initVar = 1u;
    }
    myUART_Enable();
}


/*******************************************************************************
* Function Name: myUART_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  myUART_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void myUART_Init(void) 
{
    #if(myUART_RX_ENABLED || myUART_HD_ENABLED)

        #if(myUART_RX_INTERRUPT_ENABLED && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            (void)CyIntSetVector(myUART_RX_VECT_NUM, &myUART_RXISR);
            CyIntSetPriority(myUART_RX_VECT_NUM, myUART_RX_PRIOR_NUM);
        #endif /* End myUART_RX_INTERRUPT_ENABLED */

        #if (myUART_RXHW_ADDRESS_ENABLED)
            myUART_SetRxAddressMode(myUART_RXAddressMode);
            myUART_SetRxAddress1(myUART_RXHWADDRESS1);
            myUART_SetRxAddress2(myUART_RXHWADDRESS2);
        #endif /* End myUART_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        myUART_RXBITCTR_PERIOD_REG = myUART_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        myUART_RXSTATUS_MASK_REG  = myUART_INIT_RX_INTERRUPTS_MASK;
    #endif /* End myUART_RX_ENABLED || myUART_HD_ENABLED*/

    #if(myUART_TX_ENABLED)
        #if(myUART_TX_INTERRUPT_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            (void)CyIntSetVector(myUART_TX_VECT_NUM, &myUART_TXISR);
            CyIntSetPriority(myUART_TX_VECT_NUM, myUART_TX_PRIOR_NUM);
        #endif /* End myUART_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(myUART_TXCLKGEN_DP)
            myUART_TXBITCLKGEN_CTR_REG = myUART_BIT_CENTER;
            myUART_TXBITCLKTX_COMPLETE_REG = (myUART_NUMBER_OF_DATA_BITS +
                        myUART_NUMBER_OF_START_BIT) * myUART_OVER_SAMPLE_COUNT;
        #else
            myUART_TXBITCTR_PERIOD_REG = ((myUART_NUMBER_OF_DATA_BITS +
                        myUART_NUMBER_OF_START_BIT) * myUART_OVER_SAMPLE_8) - 1u;
        #endif /* End myUART_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(myUART_TX_INTERRUPT_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))
            myUART_TXSTATUS_MASK_REG = myUART_TX_STS_FIFO_EMPTY;
        #else
            myUART_TXSTATUS_MASK_REG = myUART_INIT_TX_INTERRUPTS_MASK;
        #endif /*End myUART_TX_INTERRUPT_ENABLED*/

    #endif /* End myUART_TX_ENABLED */

    #if(myUART_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        myUART_WriteControlRegister( \
            (myUART_ReadControlRegister() & (uint8)~myUART_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(myUART_PARITY_TYPE << myUART_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End myUART_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: myUART_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  myUART_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void myUART_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if(myUART_RX_ENABLED || myUART_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        myUART_RXBITCTR_CONTROL_REG |= myUART_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        myUART_RXSTATUS_ACTL_REG  |= myUART_INT_ENABLE;
        #if(myUART_RX_INTERRUPT_ENABLED && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))
            CyIntEnable(myUART_RX_VECT_NUM);
            #if (myUART_RXHW_ADDRESS_ENABLED)
                myUART_rxAddressDetected = 0u;
            #endif /* End myUART_RXHW_ADDRESS_ENABLED */
        #endif /* End myUART_RX_INTERRUPT_ENABLED */
    #endif /* End myUART_RX_ENABLED || myUART_HD_ENABLED*/

    #if(myUART_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!myUART_TXCLKGEN_DP)
            myUART_TXBITCTR_CONTROL_REG |= myUART_CNTR_ENABLE;
        #endif /* End myUART_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        myUART_TXSTATUS_ACTL_REG |= myUART_INT_ENABLE;
        #if(myUART_TX_INTERRUPT_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))
            CyIntEnable(myUART_TX_VECT_NUM);
        #endif /* End myUART_TX_INTERRUPT_ENABLED*/
     #endif /* End myUART_TX_ENABLED */

    #if(myUART_INTERNAL_CLOCK_USED)
        /* Enable the clock. */
        myUART_IntClock_Start();
    #endif /* End myUART_INTERNAL_CLOCK_USED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: myUART_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void myUART_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if(myUART_RX_ENABLED || myUART_HD_ENABLED)
        myUART_RXBITCTR_CONTROL_REG &= (uint8)~myUART_CNTR_ENABLE;
    #endif /* End myUART_RX_ENABLED */

    #if(myUART_TX_ENABLED)
        #if(!myUART_TXCLKGEN_DP)
            myUART_TXBITCTR_CONTROL_REG &= (uint8)~myUART_CNTR_ENABLE;
        #endif /* End myUART_TXCLKGEN_DP */
    #endif /* myUART_TX_ENABLED */

    #if(myUART_INTERNAL_CLOCK_USED)
        /* Disable the clock. */
        myUART_IntClock_Stop();
    #endif /* End myUART_INTERNAL_CLOCK_USED */

    /* Disable internal interrupt component */
    #if(myUART_RX_ENABLED || myUART_HD_ENABLED)
        myUART_RXSTATUS_ACTL_REG  &= (uint8)~myUART_INT_ENABLE;
        #if(myUART_RX_INTERRUPT_ENABLED && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))
            myUART_DisableRxInt();
        #endif /* End myUART_RX_INTERRUPT_ENABLED */
    #endif /* End myUART_RX_ENABLED */

    #if(myUART_TX_ENABLED)
        myUART_TXSTATUS_ACTL_REG &= (uint8)~myUART_INT_ENABLE;
        #if(myUART_TX_INTERRUPT_ENABLED && (myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH))
            myUART_DisableTxInt();
        #endif /* End myUART_TX_INTERRUPT_ENABLED */
    #endif /* End myUART_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: myUART_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 myUART_ReadControlRegister(void) 
{
    #if( myUART_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(myUART_CONTROL_REG);
    #endif /* End myUART_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: myUART_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  myUART_WriteControlRegister(uint8 control) 
{
    #if( myUART_CONTROL_REG_REMOVED )
        if(control != 0u) { }      /* release compiler warning */
    #else
       myUART_CONTROL_REG = control;
    #endif /* End myUART_CONTROL_REG_REMOVED */
}


#if(myUART_RX_ENABLED || myUART_HD_ENABLED)

    #if(myUART_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: myUART_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void myUART_EnableRxInt(void) 
        {
            CyIntEnable(myUART_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: myUART_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void myUART_DisableRxInt(void) 
        {
            CyIntDisable(myUART_RX_VECT_NUM);
        }

    #endif /* myUART_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: myUART_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void myUART_SetRxInterruptMode(uint8 intSrc) 
    {
        myUART_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: myUART_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  myUART_rxBuffer - RAM buffer pointer for save received data.
    *  myUART_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  myUART_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  myUART_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 myUART_ReadRxData(void) 
    {
        uint8 rxData;

        #if(myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_DisableRxInt();
            #endif /* myUART_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = myUART_rxBufferRead;
            loc_rxBufferWrite = myUART_rxBufferWrite;

            if( (myUART_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = myUART_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;

                if(loc_rxBufferRead >= myUART_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                myUART_rxBufferRead = loc_rxBufferRead;

                if(myUART_rxBufferLoopDetect != 0u )
                {
                    myUART_rxBufferLoopDetect = 0u;
                    #if( (myUART_RX_INTERRUPT_ENABLED) && (myUART_FLOW_CONTROL != 0u) && \
                         (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( myUART_HD_ENABLED )
                            if((myUART_CONTROL_REG & myUART_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only in RX
                                *  configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                myUART_RXSTATUS_MASK_REG  |= myUART_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            myUART_RXSTATUS_MASK_REG  |= myUART_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end myUART_HD_ENABLED */
                    #endif /* myUART_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = myUART_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_EnableRxInt();
            #endif /* End myUART_RX_INTERRUPT_ENABLED */

        #else /* myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = myUART_RXDATA_REG;

        #endif /* myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: myUART_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  myUART_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn?t free space in
    *   myUART_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   myUART_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 myUART_ReadRxStatus(void) 
    {
        uint8 status;

        status = myUART_RXSTATUS_REG & myUART_RX_HW_MASK;

        #if(myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH)
            if( myUART_rxBufferOverflow != 0u )
            {
                status |= myUART_RX_STS_SOFT_BUFF_OVER;
                myUART_rxBufferOverflow = 0u;
            }
        #endif /* myUART_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: myUART_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error
    *  condition exists, zero is returned; otherwise, character is read and
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  myUART_rxBuffer - RAM buffer pointer for save received data.
    *  myUART_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  myUART_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  myUART_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 myUART_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_DisableRxInt();
            #endif /* myUART_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = myUART_rxBufferRead;
            loc_rxBufferWrite = myUART_rxBufferWrite;

            if( (myUART_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = myUART_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;
                if(loc_rxBufferRead >= myUART_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                myUART_rxBufferRead = loc_rxBufferRead;

                if(myUART_rxBufferLoopDetect > 0u )
                {
                    myUART_rxBufferLoopDetect = 0u;
                    #if( (myUART_RX_INTERRUPT_ENABLED) && (myUART_FLOW_CONTROL != 0u) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( myUART_HD_ENABLED )
                            if((myUART_CONTROL_REG & myUART_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only if
                                *  RX configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                myUART_RXSTATUS_MASK_REG  |= myUART_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            myUART_RXSTATUS_MASK_REG  |= myUART_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end myUART_HD_ENABLED */
                    #endif /* myUART_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }

            }
            else
            {   rxStatus = myUART_RXSTATUS_REG;
                if((rxStatus & myUART_RX_STS_FIFO_NOTEMPTY) != 0u)
                {   /* Read received data from FIFO*/
                    rxData = myUART_RXDATA_REG;
                    /*Check status on error*/
                    if((rxStatus & (myUART_RX_STS_BREAK | myUART_RX_STS_PAR_ERROR |
                                   myUART_RX_STS_STOP_ERROR | myUART_RX_STS_OVERRUN)) != 0u)
                    {
                        rxData = 0u;
                    }
                }
            }

            /* Enable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_EnableRxInt();
            #endif /* myUART_RX_INTERRUPT_ENABLED */

        #else /* myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

            rxStatus =myUART_RXSTATUS_REG;
            if((rxStatus & myUART_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO*/
                rxData = myUART_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (myUART_RX_STS_BREAK | myUART_RX_STS_PAR_ERROR |
                               myUART_RX_STS_STOP_ERROR | myUART_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        #endif /* myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: myUART_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 myUART_GetByte(void) 
    {
        return ( ((uint16)myUART_ReadRxStatus() << 8u) | myUART_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: myUART_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left
    *  in the RX buffer
    *
    * Global Variables:
    *  myUART_rxBufferWrite - used to calculate left bytes.
    *  myUART_rxBufferRead - used to calculate left bytes.
    *  myUART_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 myUART_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_DisableRxInt();
            #endif /* myUART_RX_INTERRUPT_ENABLED */

            if(myUART_rxBufferRead == myUART_rxBufferWrite)
            {
                if(myUART_rxBufferLoopDetect > 0u)
                {
                    size = myUART_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(myUART_rxBufferRead < myUART_rxBufferWrite)
            {
                size = (myUART_rxBufferWrite - myUART_rxBufferRead);
            }
            else
            {
                size = (myUART_RXBUFFERSIZE - myUART_rxBufferRead) + myUART_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_EnableRxInt();
            #endif /* End myUART_RX_INTERRUPT_ENABLED */

        #else /* myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = ((myUART_RXSTATUS_REG & myUART_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

        #endif /* End myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: myUART_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the RX RAM buffer by setting the read and write pointers both to zero.
    *  Clears hardware RX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_rxBufferWrite - cleared to zero.
    *  myUART_rxBufferRead - cleared to zero.
    *  myUART_rxBufferLoopDetect - cleared to zero.
    *  myUART_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *******************************************************************************/
    void myUART_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* clear the HW FIFO */
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        myUART_RXDATA_AUX_CTL_REG |=  myUART_RX_FIFO_CLR;
        myUART_RXDATA_AUX_CTL_REG &= (uint8)~myUART_RX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_DisableRxInt();
            #endif /* End myUART_RX_INTERRUPT_ENABLED */

            myUART_rxBufferRead = 0u;
            myUART_rxBufferWrite = 0u;
            myUART_rxBufferLoopDetect = 0u;
            myUART_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(myUART_RX_INTERRUPT_ENABLED)
                myUART_EnableRxInt();
            #endif /* End myUART_RX_INTERRUPT_ENABLED */
        #endif /* End myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH */

    }


    /*******************************************************************************
    * Function Name: myUART_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  myUART__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  myUART__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  myUART__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  myUART__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  myUART__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  myUART_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void myUART_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(myUART_RXHW_ADDRESS_ENABLED)
            #if(myUART_CONTROL_REG_REMOVED)
                if(addressMode != 0u) { }     /* release compiler warning */
            #else /* myUART_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = myUART_CONTROL_REG & (uint8)~myUART_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << myUART_CTRL_RXADDR_MODE0_SHIFT);
                myUART_CONTROL_REG = tmpCtrl;
                #if(myUART_RX_INTERRUPT_ENABLED && \
                   (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH) )
                    myUART_rxAddressMode = addressMode;
                    myUART_rxAddressDetected = 0u;
                #endif /* End myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH*/
            #endif /* End myUART_CONTROL_REG_REMOVED */
        #else /* myUART_RXHW_ADDRESS_ENABLED */
            if(addressMode != 0u) { }     /* release compiler warning */
        #endif /* End myUART_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: myUART_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void myUART_SetRxAddress1(uint8 address) 

    {
        myUART_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: myUART_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void myUART_SetRxAddress2(uint8 address) 
    {
        myUART_RXADDRESS2_REG = address;
    }

#endif  /* myUART_RX_ENABLED || myUART_HD_ENABLED*/


#if( (myUART_TX_ENABLED) || (myUART_HD_ENABLED) )

    #if(myUART_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: myUART_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void myUART_EnableTxInt(void) 
        {
            CyIntEnable(myUART_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: myUART_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void myUART_DisableTxInt(void) 
        {
            CyIntDisable(myUART_TX_VECT_NUM);
        }

    #endif /* myUART_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: myUART_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void myUART_SetTxInterruptMode(uint8 intSrc) 
    {
        myUART_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: myUART_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the
    *  bus is available. WriteTxData sends a byte without checking for buffer room
    *  or status. It is up to the user to separately check status.
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  myUART_txBuffer - RAM buffer pointer for save data for transmission
    *  myUART_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  myUART_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  myUART_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void myUART_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(myUART_initVar != 0u)
        {
            #if(myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(myUART_TX_INTERRUPT_ENABLED)
                    myUART_DisableTxInt();
                #endif /* End myUART_TX_INTERRUPT_ENABLED */

                if( (myUART_txBufferRead == myUART_txBufferWrite) &&
                    ((myUART_TXSTATUS_REG & myUART_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    myUART_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(myUART_txBufferWrite >= myUART_TXBUFFERSIZE)
                    {
                        myUART_txBufferWrite = 0u;
                    }

                    myUART_txBuffer[myUART_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    myUART_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(myUART_TX_INTERRUPT_ENABLED)
                    myUART_EnableTxInt();
                #endif /* End myUART_TX_INTERRUPT_ENABLED */

            #else /* myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                myUART_TXDATA_REG = txDataByte;

            #endif /* End myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: myUART_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to
    *  the user to handle all bits in this return value accordingly, even if the bit
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.
    *
    *******************************************************************************/
    uint8 myUART_ReadTxStatus(void) 
    {
        return(myUART_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: myUART_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_txBuffer - RAM buffer pointer for save data for transmission
    *  myUART_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  myUART_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  myUART_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void myUART_PutChar(uint8 txDataByte) 
    {
            #if(myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH)
                /* The temporary output pointer is used since it takes two instructions
                *  to increment with a wrap, and we can't risk doing that with the real
                *  pointer and getting an interrupt in between instructions.
                */
                uint8 loc_txBufferWrite;
                uint8 loc_txBufferRead;

                do{
                    /* Block if software buffer is full, so we don't overwrite. */
                    #if ((myUART_TXBUFFERSIZE > myUART_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Disable TX interrupt to protect variables that could change on interrupt */
                        CyIntDisable(myUART_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    loc_txBufferWrite = myUART_txBufferWrite;
                    loc_txBufferRead = myUART_txBufferRead;
                    #if ((myUART_TXBUFFERSIZE > myUART_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Enable interrupt to continue transmission */
                        CyIntEnable(myUART_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }while( (loc_txBufferWrite < loc_txBufferRead) ? (loc_txBufferWrite == (loc_txBufferRead - 1u)) :
                                        ((loc_txBufferWrite - loc_txBufferRead) ==
                                        (uint8)(myUART_TXBUFFERSIZE - 1u)) );

                if( (loc_txBufferRead == loc_txBufferWrite) &&
                    ((myUART_TXSTATUS_REG & myUART_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    myUART_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(loc_txBufferWrite >= myUART_TXBUFFERSIZE)
                    {
                        loc_txBufferWrite = 0u;
                    }
                    /* Add to the software buffer. */
                    myUART_txBuffer[loc_txBufferWrite] = txDataByte;
                    loc_txBufferWrite++;

                    /* Finally, update the real output pointer */
                    #if ((myUART_TXBUFFERSIZE > myUART_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntDisable(myUART_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    myUART_txBufferWrite = loc_txBufferWrite;
                    #if ((myUART_TXBUFFERSIZE > myUART_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntEnable(myUART_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }

            #else /* myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */

                while((myUART_TXSTATUS_REG & myUART_TX_STS_FIFO_FULL) != 0u)
                {
                    ; /* Wait for room in the FIFO. */
                }

                /* Add directly to the FIFO. */
                myUART_TXDATA_REG = txDataByte;

            #endif /* End myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: myUART_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: char pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole
    *  string, it will block until the entire string has been written to the
    *  transmit buffer.
    *
    *******************************************************************************/
    void myUART_PutString(const char8 string[]) 
    {
        uint16 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(myUART_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(string[buf_index] != (char8)0)
            {
                myUART_PutChar((uint8)string[buf_index]);
                buf_index++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: myUART_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void myUART_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(myUART_initVar != 0u)
        {
            do
            {
                myUART_PutChar(string[buf_index]);
                buf_index++;
            }while(buf_index < byteCount);
        }
    }


    /*******************************************************************************
    * Function Name: myUART_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void myUART_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(myUART_initVar != 0u)
        {
            myUART_PutChar(txDataByte);
            myUART_PutChar(0x0Du);
            myUART_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: myUART_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  myUART_txBufferWrite - used to calculate left space.
    *  myUART_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 myUART_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(myUART_TX_INTERRUPT_ENABLED)
                myUART_DisableTxInt();
            #endif /* End myUART_TX_INTERRUPT_ENABLED */

            if(myUART_txBufferRead == myUART_txBufferWrite)
            {
                size = 0u;
            }
            else if(myUART_txBufferRead < myUART_txBufferWrite)
            {
                size = (myUART_txBufferWrite - myUART_txBufferRead);
            }
            else
            {
                size = (myUART_TXBUFFERSIZE - myUART_txBufferRead) + myUART_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(myUART_TX_INTERRUPT_ENABLED)
                myUART_EnableTxInt();
            #endif /* End myUART_TX_INTERRUPT_ENABLED */

        #else /* myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */

            size = myUART_TXSTATUS_REG;

            /* Is the fifo is full. */
            if((size & myUART_TX_STS_FIFO_FULL) != 0u)
            {
                size = myUART_FIFO_LENGTH;
            }
            else if((size & myUART_TX_STS_FIFO_EMPTY) != 0u)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: myUART_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the TX RAM buffer by setting the read and write pointers both to zero.
    *  Clears the hardware TX FIFO.  Any data present in the FIFO will not be sent.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_txBufferWrite - cleared to zero.
    *  myUART_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void myUART_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        /* clear the HW FIFO */
        myUART_TXDATA_AUX_CTL_REG |=  myUART_TX_FIFO_CLR;
        myUART_TXDATA_AUX_CTL_REG &= (uint8)~myUART_TX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(myUART_TX_INTERRUPT_ENABLED)
                myUART_DisableTxInt();
            #endif /* End myUART_TX_INTERRUPT_ENABLED */

            myUART_txBufferRead = 0u;
            myUART_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(myUART_TX_INTERRUPT_ENABLED)
                myUART_EnableTxInt();
            #endif /* End myUART_TX_INTERRUPT_ENABLED */

        #endif /* End myUART_TXBUFFERSIZE > myUART_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: myUART_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  myUART_initVar - checked to identify that the component has been
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void myUART_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(myUART_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(myUART_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == myUART_SEND_BREAK) ||
                    (retMode == myUART_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    myUART_WriteControlRegister(myUART_ReadControlRegister() |
                                                          myUART_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    myUART_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = myUART_TXSTATUS_REG;
                    }while((tmpStat & myUART_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == myUART_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == myUART_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = myUART_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & myUART_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == myUART_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == myUART_REINIT) ||
                    (retMode == myUART_SEND_WAIT_REINIT) )
                {
                    myUART_WriteControlRegister(myUART_ReadControlRegister() &
                                                  (uint8)~myUART_CTRL_HD_SEND_BREAK);
                }

            #else /* myUART_HD_ENABLED Full Duplex mode */

                static uint8 tx_period;

                if( (retMode == myUART_SEND_BREAK) ||
                    (retMode == myUART_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode*/
                    #if( (myUART_PARITY_TYPE != myUART__B_UART__NONE_REVB) || \
                                        (myUART_PARITY_TYPE_SW != 0u) )
                        myUART_WriteControlRegister(myUART_ReadControlRegister() |
                                                              myUART_CTRL_HD_SEND_BREAK);
                    #endif /* End myUART_PARITY_TYPE != myUART__B_UART__NONE_REVB  */

                    #if(myUART_TXCLKGEN_DP)
                        tx_period = myUART_TXBITCLKTX_COMPLETE_REG;
                        myUART_TXBITCLKTX_COMPLETE_REG = myUART_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = myUART_TXBITCTR_PERIOD_REG;
                        myUART_TXBITCTR_PERIOD_REG = myUART_TXBITCTR_BREAKBITS8X;
                    #endif /* End myUART_TXCLKGEN_DP */

                    /* Send zeros*/
                    myUART_TXDATA_REG = 0u;

                    do /* wait until transmit starts */
                    {
                        tmpStat = myUART_TXSTATUS_REG;
                    }while((tmpStat & myUART_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == myUART_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == myUART_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = myUART_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & myUART_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == myUART_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == myUART_REINIT) ||
                    (retMode == myUART_SEND_WAIT_REINIT) )
                {

                    #if(myUART_TXCLKGEN_DP)
                        myUART_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        myUART_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End myUART_TXCLKGEN_DP */

                    #if( (myUART_PARITY_TYPE != myUART__B_UART__NONE_REVB) || \
                         (myUART_PARITY_TYPE_SW != 0u) )
                        myUART_WriteControlRegister(myUART_ReadControlRegister() &
                                                      (uint8)~myUART_CTRL_HD_SEND_BREAK);
                    #endif /* End myUART_PARITY_TYPE != NONE */
                }
            #endif    /* End myUART_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: myUART_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void myUART_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0u)
        {
            #if( myUART_CONTROL_REG_REMOVED == 0u )
                myUART_WriteControlRegister(myUART_ReadControlRegister() |
                                                      myUART_CTRL_MARK);
            #endif /* End myUART_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
            #if( myUART_CONTROL_REG_REMOVED == 0u )
                myUART_WriteControlRegister(myUART_ReadControlRegister() &
                                                    (uint8)~myUART_CTRL_MARK);
            #endif /* End myUART_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndmyUART_TX_ENABLED */

#if(myUART_HD_ENABLED)


    /*******************************************************************************
    * Function Name: myUART_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART.
    *
    * Side Effects:
    *  Disable RX interrupt mask, when software buffer has been used.
    *
    *******************************************************************************/
    void myUART_LoadTxConfig(void) 
    {
        #if((myUART_RX_INTERRUPT_ENABLED) && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))
            /* Disable RX interrupts before set TX configuration */
            myUART_SetRxInterruptMode(0u);
        #endif /* myUART_RX_INTERRUPT_ENABLED */

        myUART_WriteControlRegister(myUART_ReadControlRegister() | myUART_CTRL_HD_SEND);
        myUART_RXBITCTR_PERIOD_REG = myUART_HD_TXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(myUART_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */
    }


    /*******************************************************************************
    * Function Name: myUART_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    * Side Effects:
    *  Set RX interrupt mask based on customizer settings, when software buffer
    *  has been used.
    *
    *******************************************************************************/
    void myUART_LoadRxConfig(void) 
    {
        myUART_WriteControlRegister(myUART_ReadControlRegister() &
                                                (uint8)~myUART_CTRL_HD_SEND);
        myUART_RXBITCTR_PERIOD_REG = myUART_HD_RXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(myUART_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */

        #if((myUART_RX_INTERRUPT_ENABLED) && (myUART_RXBUFFERSIZE > myUART_FIFO_LENGTH))
            /* Enable RX interrupt after set RX configuration */
            myUART_SetRxInterruptMode(myUART_INIT_RX_INTERRUPTS_MASK);
        #endif /* myUART_RX_INTERRUPT_ENABLED */
    }

#endif  /* myUART_HD_ENABLED */


/* [] END OF FILE */
